package 
{
	import com.mollyjameson.leaderboardexample.gamestates.BaseGameState;
	import com.mollyjameson.leaderboardexample.gamestates.StateGame;
	import com.mollyjameson.leaderboardexample.gamestates.StateLeaderboard;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.NetStatusEvent;
	import flash.net.SharedObject;
	import flash.net.SharedObjectFlushStatus;
	
	/**
	 * Main Entry point class for storing our score
	 * @author Molly Jameson
	 */
	public class Main extends Sprite 
	{
		public static const W:int = 640;
		public static const H:int = 480;
		
		public static var Inst:Main;
		
		public static var EntryName:String = "default";
		public static var EntryScore:int = 123;
		
		// SharedObject in flash is what saves data from session to session
		private var m_FlashCookie:SharedObject;
		
		// game state data that keeps track of what to display.
		// normal games loop through a title, gameplay, and end screen at least
		private var m_CurrState:BaseGameState;
		private var m_GameStates:Object =
		{
			"state_game":new StateGame(),
			"state_leaderboard":new StateLeaderboard()
		}
		
		public function Main():void 
		{
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			
			// entry point
			Inst = this;
			
			// Create a saved "SharedObject" so that data can be saved from session to session
			// All local shared objects just need a unique name and that creates an object of key value pairs that you can use to save anything
			m_FlashCookie = SharedObject.getLocal("LeaderboardExample");
			
			// kciks off the game
			m_CurrState = m_GameStates["state_game"];
			m_CurrState.Enter();
			
			this.addEventListener(Event.ENTER_FRAME, onFrame);
			
			// If we even need to completely clear our test saved data, uncomment
			 /*for(var key:String in m_FlashCookie.data)
			 {
				delete m_FlashCookie.data[key];
			}
			m_FlashCookie.flush();*/
		}
		
		// Game loop that just updates the gamestate
		private function onFrame(ev:Event):void
		{
			m_CurrState.Update();
			if ( m_CurrState.NextState != "none" )
			{
				m_CurrState.Exit();
				var next_state:BaseGameState = m_GameStates[m_CurrState.NextState];
				if( next_state != null )
				{
					m_CurrState = next_state;
				}
				else
				{
					throw new Error("attempted to switch to invalid state " + m_CurrState.NextState );
				}
				m_CurrState.Enter();
			}
		}
		
		// Function to get saved data by a key
		public function getSaved(key:String):Object
		{
			return m_FlashCookie.data.key;
		}
		
		// Function to store data by key.
		public function saveData(key:String, dataToSave:Object):void
		{
			m_FlashCookie.data.key = dataToSave;
			 var flushStatus:String = null;
            try {
                flushStatus = m_FlashCookie.flush(10000);
            } catch (error:Error) {
                trace("Error...Could not write SharedObject to disk\n");
            }
            if (flushStatus != null) {
                switch (flushStatus) {
                    case SharedObjectFlushStatus.PENDING:
                        trace("Requesting permission to save object...\n");
                        m_FlashCookie.addEventListener(NetStatusEvent.NET_STATUS, onFlushStatus);
                        break;
                    case SharedObjectFlushStatus.FLUSHED:
						// Locally, data will just be flushed immediately.
                        trace("Value flushed to disk.\n");
                        break;
                }
            }
		}
		
		// Debugging mostly to see if something happened when trying to save data to disk.
		private function onFlushStatus(event:NetStatusEvent):void 
		{
            trace("User closed permission dialog...\n");
            switch (event.info.code) 
			{
                case "SharedObject.Flush.Success":
                    trace("User granted permission -- value saved.\n");
                    break;
                case "SharedObject.Flush.Failed":
                    trace("User denied permission -- value not saved.\n");
                    break;
            }
            m_FlashCookie.removeEventListener(NetStatusEvent.NET_STATUS, onFlushStatus);
        }
		
	}
	
}