package com.mollyjameson.leaderboardexample.gamestates 
{
	import com.mollyjameson.leaderboardexample.utils.DebugButton;
	import flash.events.MouseEvent;
	import flash.text.TextFormatAlign;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFieldAutoSize;
	
	/**
	 * ...
	 * @author Molly Jameson
	 */
	public class StateLeaderboard extends BaseGameState 
	{
		private var m_TF:TextField;
		
		// number of top scores we want to display and save.
		private const NUM_SCORES_SAVED:int = 10;
		
		// constructor is only called once when the stage is inited.
		public function StateLeaderboard() 
		{
			var myFormat:TextFormat = new TextFormat();
			myFormat.size = 24;
			myFormat.align = TextFormatAlign.CENTER;
			
			// a header.
			var tf:TextField = new TextField();
			tf.defaultTextFormat = myFormat;
			tf.mouseEnabled = false;
			tf.text = "Good job, Here is a high score list:";
			tf.wordWrap = true;
			tf.width = Main.W / 2 + 100;
			tf.autoSize = TextFieldAutoSize.CENTER;
			tf.x = Main.W / 2 - tf.width/2;
			tf.y = tf.height/2;
			this.addChild(tf);
			
			// button to restart the game
			var btn:DebugButton = new DebugButton("Play Again?");
			this.addChild(btn);
			btn.x = Main.W/2 - btn.width/2;
			btn.y = Main.H - btn.height - 100;
			btn.addEventListener(MouseEvent.CLICK, onMouseClicked);
			
			
			// The textfield that we refresh all our leaderboard data to show here.
			m_TF = new TextField();
			m_TF.width = Main.W / 2;
			m_TF.mouseEnabled = false;
			m_TF.autoSize = TextFieldAutoSize.CENTER;
			m_TF.y = tf.height + tf.y;
			m_TF.x = Main.W / 2;
			m_TF.multiline = true;
			this.addChild(m_TF);
		}
		
		// Enter is called every time we enter this gamestate after the game
		override public function Enter():void 
		{
			super.Enter();
			
			trace("Executing: StateLeaderboard::Enter");
			
			// this returns from our main class our data blob of saved scores
			var arr:Array = Main.Inst.getSaved("TopScores") as Array;
			
			// if no scores have been saved then we creat a brand new array and start saving data to it
			if ( arr == null)
			{
				arr = new Array();
			}
			
			// create the object that stores the data that just got created from our last gamestate
			// we can add multiple stats in here if we wanted, for example "enemies killed" and "coins collected"
			// in this case we just use score as an example so we have only one number to sort on
			var latest_score_object:Object = { 
											   name: Main.EntryName, 
											   score: Main.EntryScore
											 };
			// push the last score onto the list
			arr.push( latest_score_object );
			// sort the list from highest to lowest based on the score figure.
			arr.sortOn("score", Array.NUMERIC | Array.DESCENDING);
			
			
			// if adding the score from the last game made it so there were more than 10 items of the list, remove the last #11.
			// since the array was sorted on the line above that means that the last score will be in the right place regardless
			if ( arr.length > NUM_SCORES_SAVED )
			{
				arr.pop();
			}
			
			// Save our sorted array with latest data back so we can retrieve it next time
			Main.Inst.saveData("TopScores", arr);
			
			// In this example implementation we display all our data in a single textfield.
			// In a real game we'd probably work with an artist to put these on seperate lines or a grid
			var myHTML:String = "";
			
			var total_stored_scores:int = arr.length;
			for (var i:int = 0; i < total_stored_scores; ++i)
			{
				// loop through every entry, every entry has a "name" and "score" field as that's what we save.
				var leaderboard_entry:Object = arr[i];
				
				// is this the last score that was just entered last gamestate?
				if ( leaderboard_entry == latest_score_object )
				{
					myHTML += (i+1) + ". <b><font color=\"#0002E5\">"+ leaderboard_entry.name + " " + leaderboard_entry.score +"</font></b><br>";
				}
				else
				{
					myHTML += (i+1) + ". "+ leaderboard_entry.name + " " + leaderboard_entry.score +"<br>";
				}
			}
			trace("html: " + myHTML);
			m_TF.htmlText = myHTML;
			
		}
		override public function Exit():void 
		{
			super.Exit();
			trace("Executing: StateLeaderboard::Exit");
		}
		
		public function onMouseClicked(ev:MouseEvent):void
		{
			// go to next state and repeat the game
			NextState = "state_game";
		}
		
	}

}